/*
 * Segment.cpp
 *
 *  Created on: 2010.09.09.
 *      Author: Karma
 */

#include "Segment.h"

#include <FMedia.h>

using namespace Osp::Base;
using namespace Osp::Graphics;
using namespace Osp::Media;
using namespace Osp::Ui;

Bitmap *Segment::__backgrounds[8] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};

static const int FONT_SIZE = 36;

static const String BackgroundPaths[] =
{
	L"/Res/Segm_MiddleBtn_Normal_9P.png",
	L"/Res/Segm_LeftBtn_Normal_9P.png",
	L"/Res/Segm_RightBtn_Normal_9P.png",
	L"/Res/Segm_OnlyBtn_Normal_9P.png",
	L"/Res/Segm_MiddleBtn_Pressed_9P.png",
	L"/Res/Segm_LeftBtn_Pressed_9P.png",
	L"/Res/Segm_RightBtn_Pressed_9P.png",
	L"/Res/Segm_OnlyBtn_Pressed_9P.png"
};

void
Segment::PreloadBitmaps()
{
	for (int i = 0; i < 8; i++)
	{
		GetBitmap(i);
	}
}

Segment::Segment(IActionEventListener &listener) :
	__listener(listener), __caption(NULL),
	__captionElement(NULL), __position(SEGMENT_LEFT),
	__selected(false)
{
}

Segment::~Segment()
{
	delete __caption;
	delete __captionElement;
}

result
Segment::Construct(const Osp::Base::String &caption, int actionId)
{
	Panel::Construct(Rectangle(0, 0, 1, 1));
	AddTouchEventListener(*this);

	__captionElement = new TextElement;
	__captionElement->Construct(caption);


	Font font;
	font.Construct(FONT_STYLE_PLAIN, FONT_SIZE);
	__captionElement->SetFont(font);

	__caption = new EnrichedText;
	__caption->Construct(Dimension(140, 40));
	__caption->Add(*__captionElement);
	__caption->SetTextAbbreviationEnabled(true);
	__caption->SetHorizontalAlignment(TEXT_ALIGNMENT_CENTER);
	__caption->SetVerticalAlignment(TEXT_ALIGNMENT_MIDDLE);

	__actionId = actionId;
	return E_SUCCESS;
}

int
Segment::GetActionId()
{
	return __actionId;
}

void
Segment::SetSelectedState(bool selected)
{
	__selected = selected;
	RequestRedraw(true);
}

void
Segment::SetPositionId(int position)
{
	__position = position;
	RequestRedraw(true);
}

result
Segment::OnDraw()
{
	Canvas *canvas = GetCanvasN();
	int bitmapId = __position + (__selected ? 4 : 0);
	canvas->DrawNinePatchedBitmap(Rectangle(0, 0, GetWidth(), GetHeight()), *GetBitmap(bitmapId));

	Font font;
	font.Construct(FONT_STYLE_PLAIN, GetHeight() - 24);

	__captionElement->SetTextColor(__selected ? Color::COLOR_WHITE : Color::COLOR_BLACK);
	__captionElement->SetFont(font);
	__caption->SetSize(GetWidth() - 8, GetHeight() - 24);

	canvas->DrawText(Point(8, 12), *__caption);
	canvas->Show();
	delete canvas;
	return E_SUCCESS;
}

Bitmap *
Segment::GetBitmap(int id)
{
	TryReturn(id >= 0 && id < 8, NULL, "Segment Bitmap ID out of range! %d", id);

	if (!__backgrounds[id])
	{
		Image image;
		image.Construct();

		__backgrounds[id] = image.DecodeN(BackgroundPaths[id], BITMAP_PIXEL_FORMAT_ARGB8888);
		result r = GetLastResult();
		TryReturn(r == E_SUCCESS, NULL, "Failed to decode bitmap %d (path: %S) [%S]", id, BackgroundPaths[id].GetPointer(), GetErrorMessage(r));
		AppLog("Loaded bitmap %d!", id);
	}

	return __backgrounds[id];
}

void
Segment::OnTouchDoublePressed(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}

void
Segment::OnTouchFocusIn(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}

void
Segment::OnTouchFocusOut(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}

void
Segment::OnTouchLongPressed(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}

void
Segment::OnTouchMoved(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}

void
Segment::OnTouchPressed(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
	__listener.OnActionPerformed(*this, __actionId);
}

void
Segment::OnTouchReleased(const Osp::Ui::Control &source, const Osp::Graphics::Point &currentPosition, const Osp::Ui::TouchEventInfo &touchInfo)
{
}
