/*
 * ToolBar.cpp
 *
 *  Created on: 2010.09.07.
 *      Author: Karma
 */

#include "ToolBar.h"

#include <FMedia.h>

using namespace Osp::Base;
using namespace Osp::Base::Collection;
using namespace Osp::Graphics;
using namespace Osp::Media;
using namespace Osp::Ui;
using namespace Osp::Ui::Controls;

Bitmap *ToolBar::__bg(NULL);

ToolBar::ToolBar()
{
}

ToolBar::~ToolBar()
{
	__buttons.RemoveAll(false);
	__listeners.RemoveAll();
}

result
ToolBar::Construct(const Rectangle &bounds)
{
	result r = __listeners.Construct(4);
	if (r == E_SUCCESS)
	{
		r = Panel::Construct(bounds);
	}

	return r;
}

Bitmap *
ToolBar::GetBackgroundBitmap()
{
	if (!__bg)
	{
		result r;
		Image image;
		r = image.Construct();
		TryReturn(r == E_SUCCESS, NULL, "Image.Construct failed [%S]", GetErrorMessage(r));

		Bitmap *tmp = image.DecodeN(L"/Res/ToolBarBg9P.png", BITMAP_PIXEL_FORMAT_ARGB8888);
		if (tmp)
		{
			__bg = tmp;
		}
		else
		{
			TryReturn(false, NULL, "Image.DecodeN failed [%S]", GetErrorMessage(GetLastResult()));
		}
	}

	return __bg;
}

result
ToolBar::OnDraw()
{
	result r = E_SUCCESS;
	Canvas *pCanvas = GetCanvasN();
	if (pCanvas)
	{
		r = pCanvas->DrawNinePatchedBitmap(GetBounds(), *GetBackgroundBitmap());
		delete pCanvas;
	}
	else
	{
		r = GetLastResult();
	}

	return r;
}

result
ToolBar::AddButton(Button &button)
{
	result r = E_SUCCESS;

	button.AddActionEventListener(*this);
	r = __buttons.Add(button);
	TryCatch(r == E_SUCCESS,
			button.RemoveActionEventListener(*this),
			"Toolbar::AddButton: __buttons.Add failed [%S]", GetErrorMessage(r));
	r = AddControl(button);
	TryCatch(r == E_SUCCESS,
			{__buttons.Remove(button, false); button.RemoveActionEventListener(*this); },
			"Toolbar::AddButton: AddControl failed [%S]", GetErrorMessage(r));
	LayoutButtons();

CATCH:
	return r;
}

void
ToolBar::LayoutButtons()
{
	Rectangle bounds = GetBounds();

	// add some padding
	bounds.x += 12;
	bounds.width -= 24;
	bounds.y += 8;
	bounds.height -= 16;

	int count = __buttons.GetCount();
	bounds.width /= count;

	for (int i = 0; i < count; i++)
	{
		Button *button = static_cast<Button *>(__buttons.GetAt(i));
		button->SetBounds(bounds);
		bounds.x += bounds.width;
	}
}

void
ToolBar::AddActionEventListener(Osp::Ui::IActionEventListener &listener)
{
	__listeners.Add(&listener);
}

void
ToolBar::RemoveActionEventListener(Osp::Ui::IActionEventListener &listener)
{
	__listeners.Remove(&listener);
}

void
ToolBar::OnActionPerformed (const Osp::Ui::Control &source, int actionId)
{
	for (int i = 0; i < __listeners.GetCount(); i++)
	{
		IActionEventListener *listener = NULL;
		__listeners.GetAt(i, listener);
		listener->OnActionPerformed(source, actionId);
	}
}
